<?php

require_once __DIR__ . '/vendor/autoload.php';

if (!defined('MODULE_PAYMENT_PAYPAY_CORE_CONFIRMATION')) {
    include DIR_FS_CATALOG . 'includes/languages/' . $language . '/modules/payment/paypayuecore.php';
}

class paypayuecore
{
    public $code, $title, $description, $enabled;

    public function paypayuecore()
    {
        global $order, $customer_id;

        $this->code = 'paypayuecore';
        $this->title = 'PayPay - Core Module';
        $this->description = MODULE_PAYMENT_PAYPAY_CORE_DESCRIPTION;
        $this->codigo = MODULE_PAYMENT_PAYPAY_CORE_CODIGO_PLATAFORMA;
        $this->chave = MODULE_PAYMENT_PAYPAY_CORE_CHAVE;
        $this->nif = MODULE_PAYMENT_PAYPAY_CORE_NIF;
        $this->email_footer = "";
        $this->sort_order = 0;
        $this->enabled = ((MODULE_PAYMENT_PAYPAY_CORE_STATUS == 'Sim') ? true : false);
        $this->order_status = 1; // Pending

        if (is_object($order)) {
            $this->update_status();
        }
    }

    public function update_status()
    {
    }

    private function getClient()
    {
        global $languages_id;
        $lang_query = tep_db_query(
            "select code from " . TABLE_LANGUAGES . " where languages_id = '" . (int) $languages_id . "'"
        );
        $lang = tep_db_fetch_array($lang_query);

        $language_code = strtoupper($lang['code']);
        switch ($language_code) {
            case 'EN':
            case 'ES':
            case 'PT':
                break;

            default:
                $language_code = strtoupper(DEFAULT_LANGUAGE);
        }

        $config = \PayPay\Configuration::setup(
            array(
                'environment'  => MODULE_PAYMENT_PAYPAY_CORE_ENVIRONMENT === 'Produção' ? 'production': defined('PAYPAY_WEBSERVICE_URL') ? 'development': 'testing',
                'platformCode' => MODULE_PAYMENT_PAYPAY_CORE_CODIGO_PLATAFORMA,
                'privateKey'   => MODULE_PAYMENT_PAYPAY_CORE_CHAVE,
                'clientId'     => MODULE_PAYMENT_PAYPAY_CORE_NIF, // usually the client NIF
                'langCode'     => $language_code,
                'platformInfo' => array(
                    'osCommerce' => tep_get_version()
                )
            )
        );

        return \PayPay\PayPayWebservice::init($config);
    }

    public function javascript_validation()
    {
        return false;
    }

    public function selection()
    {
        global $order;

        $paymentMethodCode = $this->getPaymentMethod();

        if (!$paymentMethodCode) {
            return;
        }

        if (!isset($order->paymentOptions)) {
            $client = $this->getClient();

            $requestReference = new \PayPay\Structure\RequestReferenceDetails(
                array(
                    'amount' => number_format($order->info['total'], 2, '', ''),
                )
            );

            try {
                $response = $client->validatePaymentReference($requestReference);
                $order->paymentOptions = $response->paymentOptions;
            } catch (Exception $e) {
                return;
            }
        }

        foreach ($order->paymentOptions as $po) {
            if ($po->code === $paymentMethodCode) {
                return array(
                    'id'     => $this->code,
                    'module' => 'PayPay - ' . ($po->name),
                );
            }
        }
    }

    public function confirmation()
    {
        global $language;

        $title = sprintf(MODULE_PAYMENT_PAYPAY_CORE_CONFIRMATION, $this->title);
        return array('title' => $title);
    }

    public function pre_confirmation_check()
    {
        return false;
    }

    public function process_button()
    {
        return false;
    }

    public function before_process()
    {
        global $order, $cart;
        $info = $order->info;
        $customer = $order->customer;
        $billing = $order->billing;
        $shipping = $order->shipping;

        $buyer = new \PayPay\Structure\RequestBuyerInfo(
            array(
                'lastName'   => $customer['lastname'],
                'firstName'  => $customer['firstname'],
                'customerId' => '',
                'email'      => $customer['email_address']
            )
        );

        $billingAddress = new \PayPay\Structure\RequestBillingAddress(
            array(
                'country'   => $billing['country']['iso_code_2'],
                'state'     => tep_get_zone_code($billing['country']['id'], $billing['zone_id'], null),
                'stateName' => tep_get_zone_name($billing['country']['id'], $billing['zone_id'], $billing['state']),
                'city'      => $billing['city'],
                'street1'   => $billing['street_address'],
                'street2'   => $billing['suburb'],
                'postCode'  => $billing['postcode']
            )
        );

        $shippingAddress = new \PayPay\Structure\RequestShippingAddress(
            array(
                'country'   => $shipping['country']['iso_code_2'],
                'state'     => tep_get_zone_code($shipping['country']['id'], $shipping['zone_id'], null),
                'stateName' => tep_get_zone_name($shipping['country']['id'], $shipping['zone_id'], $shipping['state']),
                'city'      => $shipping['city'],
                'street1'   => $shipping['street_address'],
                'street2'   => $shipping['suburb'],
                'postCode'  => $shipping['postcode']
            )
        );

        try {
            $requestOrder = new \PayPay\Structure\RequestPaymentOrder(
                array(
                    'amount' => number_format($info['total'], 2, '', '')
                )
            );
            $requestPayment = new \PayPay\Structure\RequestCreditCardPayment(
                $requestOrder,
                tep_href_link(FILENAME_CHECKOUT_SUCCESS),
                tep_href_link('ext/modules/payment/paypayue/redirect.php', '', 'SSL', false),
                $this->getPaymentMethod(),
                $buyer,
                $billingAddress,
                $shippingAddress
            );

            $client = $this->getClient();

            $response = $client->doWebPayment($requestPayment);

            $sql = "INSERT paypay_payment (paymentId, paymentStatus, paypayLink, token) VALUES (%d, %d, '%s', '%s')";

            $sql = sprintf($sql, $response->idTransaction, 1, $response->url, $response->token);

            $order->paypayId = $response->idTransaction;

            tep_db_query($sql);

            $this->email_footer = MODULE_PAYMENT_PAYPAY_CORE_PAYMENT_LINK . $response->url;
        } catch (Exception $e) {
            tep_redirect(
                tep_href_link(
                    FILENAME_CHECKOUT_PAYMENT,
                    'error_message=' . urlencode($e->getMessage()),
                    'SSL'
                )
            );
        }

        return false;
    }

    public function after_process()
    {
        global $insert_id, $cart, $order;

        $sql_data_array = array(
            'orders_id'         => $insert_id,
            'orders_status_id'  => $order->info['order_status'],
            'date_added'        => 'now()',
            'customer_notified' => 0,
            'comments'          => 'PayPay ID: '.$order->paypayId
        );
        tep_db_perform(TABLE_ORDERS_STATUS_HISTORY, $sql_data_array);

        $payment_query = tep_db_query(
            "SELECT id, paypayLink FROM paypay_payment WHERE paymentId = '".(int)$order->paypayId."'"
        );

        $pay = tep_db_fetch_array($payment_query);

        $sql_paypay_array = array(
            'orderId' => $insert_id,
        );

        tep_db_perform('paypay_payment', $sql_paypay_array, 'update', 'id = '.$pay['id']);

        $cart->reset(true);

        tep_redirect($pay['paypayLink']);

        return false;
    }

    protected function getPaymentMethod()
    {
        return false;
    }

    public function get_error()
    {
        return false;
    }

    public function check()
    {
        if (!isset($this->_check)) {
            $check_query = tep_db_query(
                "SELECT configuration_value
                FROM " . TABLE_CONFIGURATION . "
                WHERE configuration_key = 'MODULE_PAYMENT_PAYPAY_CORE_STATUS'");
            $this->_check = tep_db_num_rows($check_query);
        }
        return $this->_check;
    }

    public function install()
    {
        tep_db_query("CREATE TABLE IF NOT EXISTS `paypay_payment` (
          `id` INT NOT NULL AUTO_INCREMENT,
          `orderId` INT NOT NULL,
          `paymentId` INT NOT NULL,
          `paymentStatus` INT NOT NULL DEFAULT 1,
          `paymentDate` DATETIME NULL,
          `paypayLink` VARCHAR(250) NOT NULL,
          `token` VARCHAR(50) NOT NULL,
          `dateCreated` TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
          `dateUpdated` TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
          `bDeleted` TINYINT(1) NULL DEFAULT 0,
          PRIMARY KEY (`id`) )
        ENGINE = InnoDB
        DEFAULT CHARACTER SET = utf8");

        tep_db_query(
            "INSERT INTO " . TABLE_CONFIGURATION . "
                (configuration_title,
                configuration_key,
                configuration_value,
                configuration_description,
                configuration_group_id,
                sort_order,
                set_function,
                date_added)
                VALUES
                    ('Activar o módulo',
                    'MODULE_PAYMENT_PAYPAY_CORE_STATUS',
                    'Sim',
                    '',
                    '6',
                    '1',
                    'tep_cfg_select_option(array(\'Sim\', \'Não\'), ',
                    now())"
        );
        tep_db_query(
            "INSERT INTO " . TABLE_CONFIGURATION . "
                (configuration_title,
                 configuration_key,
                 configuration_value,
                 configuration_description,
                 configuration_group_id,
                 sort_order,
                 date_added)
            VALUES ('NIF de cliente',
                'MODULE_PAYMENT_PAYPAY_CORE_NIF',
                '',
                'NIF de cliente',
                '6',
                '2',
                now())"
        );
        tep_db_query(
            "INSERT INTO " . TABLE_CONFIGURATION . "
                (configuration_title,
                 configuration_key,
                 configuration_value,
                 configuration_description,
                 configuration_group_id,
                 sort_order,
                 date_added)
            VALUES
                ('Código da plataforma',
                'MODULE_PAYMENT_PAYPAY_CORE_CODIGO_PLATAFORMA',
                '',
                'Este é o código de 4 digitos que irá obter na PAYPAY quando registar a plataforma.',
                '6',
                '3',
                now())"
        );
        tep_db_query(
            "INSERT INTO " . TABLE_CONFIGURATION . "
                (configuration_title,
                 configuration_key,
                 configuration_value,
                 configuration_description,
                 configuration_group_id,
                 sort_order,
                 date_added)
            VALUES ('Chave de encriptação',
                    'MODULE_PAYMENT_PAYPAY_CORE_CHAVE',
                    '',
                    'Chave de encriptação da plataforma',
                    '6',
                    '4',
                    now())"
        );
        tep_db_query(
            "INSERT INTO " . TABLE_CONFIGURATION . "
                (configuration_title,
                 configuration_key,
                 configuration_value,
                 configuration_description,
                 configuration_group_id,
                 sort_order,
                 set_function,
                 date_added)
            VALUES ('Ambiente',
                    'MODULE_PAYMENT_PAYPAY_CORE_ENVIRONMENT',
                    'Testes',
                    '',
                    '6',
                    '5',
                    'tep_cfg_select_option(array(\'Testes\', \'Produção\'), ',
                    now())"
        );
    }

    public function remove()
    {
        tep_db_query(
            "DELETE FROM " . TABLE_CONFIGURATION . "
            WHERE configuration_key IN ('" . implode("', '", $this->keys()) . "')"
        );
    }

    public function keys()
    {
        return array(
            'MODULE_PAYMENT_PAYPAY_CORE_STATUS',
            'MODULE_PAYMENT_PAYPAY_CORE_NIF',
            'MODULE_PAYMENT_PAYPAY_CORE_CODIGO_PLATAFORMA',
            'MODULE_PAYMENT_PAYPAY_CORE_CHAVE',
            'MODULE_PAYMENT_PAYPAY_CORE_ENVIRONMENT',
        );
    }
}
